from service import DogService, CatService


class Menu:
    def __init__(self, options={}):
        self.options = options.copy()
        self.options[0] = "Exit"

    def print_options(self):
        keys = sorted(self.options.keys())
        for k in keys:
            print("{}. {}".format(k, self.options.get(k)))
        try:
            selected = int(input("What do you want to do? "))
        except ValueError:
            print("Please provide a valid option.")
            selected = None
        return selected


class AnimalSelectionMenu(Menu):
    def __init__(self, dog_callback, cat_callback):
        Menu.__init__(self, {
            1: "Dog",
            2: "Cat"
        })
        self.dog_callback = dog_callback
        self.cat_callback = cat_callback

    def start(self):
        selected = self.print_options()
        if selected == 1:
            self.dog_callback()
        elif selected == 2:
            self.cat_callback()


class GuestCheckInMenu:
    def __init__(self, dog_service, cat_service):
        self.dog_service = dog_service
        self.cat_service = cat_service

    def create_dog(self):
        self.dog_service.store(input("breed: "), input("estimated age: "))

    def create_cat(self):
        self.cat_service.store(input("breed: "), input("estimated age: "))

    def start(self):
        AnimalSelectionMenu(self.create_dog, self.create_cat).start()


class GuestCheckOutMenu:
    def __init__(self, dog_service, cat_service):
        self.dog_service = dog_service
        self.cat_service = cat_service

    def check_out_dog(self):
        ListMenu(self.dog_service, self.cat_service).list_dogs()
        id = int(input("dog id: "))
        self.dog_service.remove(id)

    def check_out_cat(self):
        ListMenu(self.dog_service, self.cat_service).list_cats()
        id = int(input("cat id: "))
        self.cat_service.remove(id)

    def start(self):
        AnimalSelectionMenu(self.check_out_dog, self.check_out_cat).start()


class ListMenu:
    def __init__(self, dog_service, cat_service):
        self.dog_service = dog_service
        self.cat_service = cat_service

    def list_dogs(self):
        sorted_dogs = sorted(self.dog_service.all(), key=lambda d: d.get_id())
        dogs = map(lambda dog: "{}) {} - {} - {} - {}".format(dog.get_id(), dog.get_admission_date(), dog.get_breed(),
                                                              dog.get_estimated_age(), ",".join(dog.get_notes())),
                   sorted_dogs)
        for dog in dogs:
            print(dog)

    def list_cats(self):
        sorted_cats = sorted(self.cat_service.all(), key=lambda c: c.get_id())
        cats = map(lambda cat: "{}) {} - {} - {} - {}".format(cat.get_id(), cat.get_admission_date(), cat.get_breed(),
                                                              cat.get_estimated_age(), ",".join(cat.get_notes())),
                   sorted_cats)
        for cat in cats:
            print(cat)

    def start(self):
        AnimalSelectionMenu(self.list_dogs, self.list_cats).start()


class AddNotesMenu:
    def __init__(self, dog_service, cat_service):
        self.dog_service = dog_service
        self.cat_service = cat_service

    def add_notes_dog(self):
        ListMenu(self.dog_service, self.cat_service).list_dogs()
        id = int(input("dog id: "))
        note = input("note: ")
        self.dog_service.get_by_id(id).add_note(note)

    def add_notes_cat(self):
        ListMenu(self.dog_service, self.cat_service).list_cats()
        id = int(input("cat id: "))
        note = input("note: ")
        self.cat_service.get_by_id(id).add_note(note)

    def start(self):
        AnimalSelectionMenu(self.add_notes_dog, self.add_notes_cat).start()


class MainMenu(Menu):
    def __init__(self, dog_service=DogService(), cat_service=CatService()):
        Menu.__init__(self, {
            1: "Guest Check-In",
            2: "Guest Check-Out",
            3: "Guests List",
            4: "Add Notes"
        })
        self.dog_service = dog_service
        self.cat_service = cat_service

    def check_in(self):
        GuestCheckInMenu(self.dog_service, self.cat_service).start()

    def check_out(self):
        GuestCheckOutMenu(self.dog_service, self.cat_service).start()

    def list(self):
        ListMenu(self.dog_service, self.cat_service).start()

    def add_notes(self):
        AddNotesMenu(self.dog_service, self.cat_service).start()

    def start(self):
        print("Welcome to Puppy Place's system!")

        selected = None
        while selected != 0:

            selected = self.print_options()

            if selected == 1:
                self.check_in()
            elif selected == 2:
                self.check_out()
            elif selected == 3:
                self.list()
            elif selected == 4:
                self.add_notes()
